<?php namespace EwTools\Handler;

use EwTools\Support\Str;

/**
 * OpenContentQueryBuilder
 *
 * @link    http://infomaker.se
 * @package EwTools\Handler
 * @since   EwTools\Handler\OpenContentQueryBuilder 0.1
 */
class OpenContentQueryBuilder {
    
    /**
     * Contains the query to use against Open Content
     *
     * @var
     */
    private $query = '';
    
    /**
     * Contains a string to be used as text-search and will therefor be escaped
     *
     * @var string
     */
    private $text = '';
    
    private function __construct( $query ) {
        $this->query = $query;
    }
    
    /**
     * Start a query
     *
     * @param $query
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public static function query( $query ) {
        return new static( $query );
    }
    
    /**
     * Start a query by adding text
     *
     * @param $text
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public static function text( $text ) {
        return ( new static( '' ) )->setText( $text );
    }
    
    public function andIf($query) {
        return $this->addQuery(" AND {$query}");
    }
    
    public function andIfNot($query) {
        return $this->addQuery(" AND NOT {$query}");
    }
    
    public function orIf($query) {
        return $this->addQuery(" OR {$query}");
    }
    
    public function orIfNot($query) {
        return $this->addQuery(" OR NOT {$query}");
    }
    
    /**
     * Add to the text
     * @param $text
     *
     * @since 0.1
     * @return OpenContentQueryBuilder
     */
    public function setText( $text ) {
        $this->text = $text;
        return $this;
    }
    
    /**
     * Append a query to the string
     *
     * @param $string
     *
     * @since 0.1
     * @return $this
     */
    public function addQuery( $string ) {
        $this->query = Str::append(trim( $string ), $this->query, " ");
        return $this;
    }
    
    private function getSanitizedText() {
        $text = stripcslashes( trim( $this->text ) );
        
        return Str::notEmpty($text) ? '"'. addslashes($text) . '"' : $text;
    }
    
    /**
     * Create a Solr-property string
     *
     * @uses  Str::contains()
     *
     * @param $name
     * @param $value
     *
     * @since 0.1
     * @return string
     */
    public static function createProperty( $name, $value ) {
        $value = Str::contains( $value, " " ) ? '"' . $value . '"' : $value;
        
        return "{$name}:{$value}";
    }
    
    /**
     * Creates a Solr-query of one or more properties with the option to change the separator
     *
     * @param string $property The name of the property to match with
     * @param string|array  $values One or a list of values to look for
     * @param string $glue
     *
     * @since 0.1
     * @return string
     */
    public static function propertySearch( $property, $values, $glue = 'OR' ) {
        $values = array_unique( array_filter( (array) $values ) );
        
        if(! empty( $values ) ) {
            $glue = trim( $glue ); // Make sure that there is no space around the glue
            
            $block = implode( " {$glue} ", array_map( function ( $value ) use ( $property ) {
                return static::createProperty( $property, $value );
            }, $values ) );
            
            return count( $values ) > 1 ? "({$block})" : $block;
        }
        
        return '';
    }
    
    public function getQuery() {
        $text = $this->getSanitizedText();
        return  Str::append(trim($this->query), $text, " ");
    }
}