<?php
/**
 * Support: for creating simple but powerful objects out of arrays for easy management
 */
namespace EwTools\Support;

use ArrayAccess;
use JsonSerializable;
use EwTools\Interfaces\Support\Jsonable;
use EwTools\Interfaces\Support\Arrayable;

/**
 * Fluent
 *
 * @link    http://infomaker.se
 * @package EwTools\Support
 * @since   EwTools\Support\Fluent 0.1
 */
class Fluent implements ArrayAccess, Arrayable, Jsonable, JsonSerializable {
	
	/**
	 * All of the attributes set on the container.
	 *
	 * @var array
	 */
	protected $attributes = [];
	
	/**
	 * Create a new fluent container instance.
	 *
	 * @param array|object $attributes
	 */
	public function __construct( $attributes = [] ) {
		$this->fill( $attributes );
	}
	
	/**
	 * Fill up attributes dynamically on the model
	 *
	 * @param array $attributes
	 *
	 * @since 0.1
	 * @return void
	 */
	protected function fill( array $attributes = [] ) {
		foreach ( $attributes as $key => $value ) {
			$this->set( $key, $value );
		}
	}
	
	/**
	 * Get an attribute from the container.
	 *
	 * @param string $key
	 * @param mixed  $default
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public function get( $key, $default = null ) {
		if ( array_key_exists( $key, $this->attributes ) ) {
			return $this->attributes[ $key ];
		}
		
		return ev_value( $default );
	}
	
	/**
	 * Get an attribute from the container.
	 *
	 * @param string $key
	 * @param mixed  $value
	 *
	 * @since 0.1
	 * @return void
	 */
	public function set( $key, $value ) {
		$this->attributes[ $key ] = $value;
	}
	
	/**
	 * Get the attributes from the container.
	 *
	 * @since 0.1
	 * @return array
	 */
	public function getAttributes() {
		return $this->attributes;
	}
	
	/**
	 * Convert the Fluent instance to an array.
	 *
	 * @since 0.1
	 * @return array
	 */
	public function toArray() {
		return $this->attributes;
	}
	
	/**
	 * Convert the object into something JSON serializable.
	 *
	 * @since 0.1
	 * @return array
	 */
	public function jsonSerialize() {
		return $this->toArray();
	}
	
	/**
	 * Convert the Fluent instance to JSON.
	 *
	 * @param int $options
	 *
	 * @since 0.1
	 * @return string
	 */
	public function toJson( $options = 0 ) {
		return json_encode( $this->jsonSerialize(), $options );
	}
	
	/**
	 * Determine if the given offset exists.
	 *
	 * @param string $offset
	 *
	 * @since 0.1
	 * @return bool
	 */
	public function offsetExists( $offset ) {
		return isset( $this->{$offset} );
	}
	
	/**
	 * Get the value for a given offset.
	 *
	 * @param string $offset
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public function offsetGet( $offset ) {
		return $this->{$offset};
	}
	
	/**
	 * Set the value at the given offset.
	 *
	 * @param string $offset
	 * @param mixed  $value
	 *
	 * @since 0.1
	 * @return void
	 */
	public function offsetSet( $offset, $value ) {
		$this->{$offset} = $value;
	}
	
	/**
	 * Unset the value at the given offset.
	 *
	 * @param string $offset
	 *
	 * @since 0.1
	 * @return void
	 */
	public function offsetUnset( $offset ) {
		unset( $this->{$offset} );
	}
	
	/**
	 * Handle dynamic calls to the container to set attributes.
	 *
	 * @param string $method
	 * @param array  $parameters
	 *
	 * @since 0.1
	 * @return $this
	 */
	public function __call( $method, $parameters ) {
		$this->attributes[ $method ] = count( $parameters ) > 0 ? $parameters[0] : true;
		
		return $this;
	}
	
	/**
	 * Dynamically retrieve the value of an attribute.
	 *
	 * @param string $key
	 *
	 * @since 0.1
	 * @return mixed
	 */
	public function __get( $key ) {
		return $this->get( $key );
	}
	
	/**
	 * Dynamically set the value of an attribute.
	 *
	 * @param string $key
	 * @param mixed  $value
	 *
	 * @since 0.1
	 * @return void
	 */
	public function __set( $key, $value ) {
		$this->set( $key, $value );
	}
	
	/**
	 * Dynamically check if an attribute is set.
	 *
	 * @param string $key
	 *
	 * @since 0.1
	 * @return bool
	 */
	public function __isset( $key ) {
		return isset( $this->attributes[ $key ] );
	}
	
	/**
	 * Dynamically unset an attribute.
	 *
	 * @param string $key
	 *
	 * @since 0.1
	 * @return void
	 */
	public function __unset( $key ) {
		unset( $this->attributes[ $key ] );
	}
}
