<?php namespace EwTools;

use EwTools\Admin\ProjectPluginPage\PluginsPage;
use EwTools\Admin\ProjectPluginPage\SettingsPage;
use EwTools\Admin\ProjectPluginPage\WidgetsPage;
use EwTools\Admin\SettingsPage\SectionsPage;
use EwTools\Admin\SettingsPage\SettingsParametersPage;
use EwTools\Interfaces\ComponentsPageInterface;
use EwTools\Interfaces\PluginConfigInterface;
use EwTools\Interfaces\PluginPageInterface;
use EwTools\Interfaces\SettingsPageInterface;
use EwTools\Storage\ObjectDB;
use EwTools\Support\Str;

/**
 * projectPlugin
 *
 * @link    http://infomaker.se
 * @package EwTools
 * @since   EwTools\projectPlugin 0.1
 */
abstract class ProjectPlugin {
    
    /**
     * Instance of this class
     *
     * @var static
     */
    protected static $instance = null;
    
    /**
     * Contains the version of this class
     *
     * @var string
     */
    protected static $version = '0.1';
    
    /**
     * Contains an array of links to be applied to the list of links to display on the plugins page
     *
     * @var array
     */
    private $actionlinks = [];
    
    /**
     * Contains an instance of the settings-page
     *
     * @var SettingsPage
     */
    protected $settings_page;
    
    /**
     * Contains the pages added to the plugin
     *
     * @var ComponentsPageInterface[]
     */
    protected $components_pages;
    
    /**
     * Contains the config class
     *
     * @var PluginConfigInterface
     */
    protected $config;
    
    /**
     * Contains the pages added to the plugin
     *
     * @var PluginPageInterface[]
     */
    protected $pages = [];
    
    /**
     * Contains the text domain for the project-plugin. Defaults to everywares textdomain
     *
     * @var string
     */
    protected $text_domain = 'ewtools_textdomain';
    
    /**
     * Get instance of this class
     *
     * @return static
     */
    public static function getInstance() {
        if( static::$instance === null ) {
            static::$instance = new static;
        }
        
        return static::$instance;
    }
    
    /**
     * Initialize Plugin
     *
     * @param ProjectPluginConfig $config
     *
     * @since 0.1
     * @return void
     */
    public function init( ProjectPluginConfig $config ) {
        
        $this->config        = $config;
    
        $this->settings_page = new SettingsPage( $this->config->getSlug() );
        $this->addPage(new PluginsPage( $this->config->getPath( 'plugins' ) ));
        $this->addPage(new WidgetsPage( $this->config->getPath( 'widgets' ) ));
        $this->addSettingsTab( new SettingsParametersPage() );
        $this->addSettingsTab( new SectionsPage() );
        $this->addActionLink( __( 'Settings', $this->text_domain ), admin_url( 'admin.php?page=' . $this->config->getSlug() ) );
        
        // Load all shortcodes
        foreach ( glob( $config->getPath( "shortcodes/*.php" ) ) as $filename ) {
            require_once( $filename );
        }
        
        $this->setupAdmin();
        
        // Add settings link for this plugin in WordPress plugin
        add_filter( "plugin_action_links_" . $this->config->getPath( 'bootstrap.php' ), function($links) { return $this->applyActionLinks($links); } );
        
        // Hooks for activation and deactivation
        register_activation_hook( $config->getPath( 'bootstrap.php' ), function() { $this->activate(); } );
        register_deactivation_hook( $config->getPath( 'bootstrap.php' ), function() { $this->deactivate(); } );
    }
    
    /**
     * Bootstrap the plugin with the necessary information from the project bootstrap.php
     *
     * @param ProjectPluginConfig $config
     *
     * @since 0.1
     * @return static
     */
    public static function bootstrap( ProjectPluginConfig $config ) {
        $plugin = static::getInstance();
        
        $plugin->init( $config );
        
        if( is_admin() ) {
            add_action( 'plugins_loaded', [ $plugin, 'getInstance' ] );
        }
        
        return $plugin;
    }
    
    /**
     * Add components page
     *
     * @param ComponentsPageInterface $page
     *
     * @since 0.1
     * @return static
     */
    protected function addComponentsPage( ComponentsPageInterface $page ) {
        $this->components_pages[] = $page;
        return $this;
    }
    
    /**
     * Add links to be applied to the list of links to display on the plugins page
     *
     * @param string $link_title
     * @param string $link_url
     * @param string $link_slug
     *
     * @since 0.1
     * @return void
     */
    public function addActionLink( $link_title, $link_url, $link_slug = '' ) {
        if( Str::isEmpty( $link_slug ) ) {
            $link_slug = Str::slug( $link_title );
        }
        $this->actionlinks[ $link_slug ] = "<a href=\"{$link_url}\">{$link_title}</a>";
    }
    
    /**
     * Add page to the list that will be added to the plugin menu
     *
     * @param PluginPageInterface $page
     *
     * @since 0.1
     * @return static
     */
    public function addPage( PluginPageInterface $page ) {
        
        if ( $page instanceof ComponentsPageInterface ) {
            $this->addComponentsPage($page);
        }
        
        $this->pages[] = $page;
        
        return $this;
    }
    
    /**
     * Create a Settings-tab from the page sent into the function
     *
     * @param SettingsPageInterface $page
     *
     * @since 0.1
     * @return void
     */
    public function addSettingsTab( SettingsPageInterface $page ) {
        $this->settings_page->addSettingsTab( $page );
    }
    
    /**
     * Retrieve a page url using the plugin page
     *
     * @param PluginPageInterface $page
     *
     * @since 0.1
     * @return string
     */
    public function getPageUrl( PluginPageInterface $page ) {
        return admin_url( 'admin.php?page=' . $page->getSlug() );
    }
    
    /**
     * Setup the Plugin for use with subpages and tabs for settings page
     *
     * @since 0.1
     * @return void
     */
    protected function setupAdmin() {
    
        // Add menu items
        add_action( 'admin_menu', function() {
            
            $menu_slug = $this->config->getSlug();
            
            // Add head menu item
            add_menu_page( $this->config->getName(), $this->config->getName(), 'manage_options', $menu_slug );
            
            // Add Settings menu item
            $this->settings_page->addToMenu( $menu_slug );
            
            foreach ( $this->pages as $page ) {
                $page->addToMenu( $menu_slug );
            }
            
        }, 10 );
        
    }
    
    /**
     * Retrieve all "Plugins" activated in Wordpress from the options table
     *
     * @since 0.1
     * @return mixed
     */
    public static function getAllPlugins() {
        return ObjectDB::get( 'active_plugins', [] );
    }
    
    // Hooks/filters:
    // ======================================================
    
    /**
     * Apply to the list of links to display on the plugins page (beside the activate/deactivate links)
     *
     * @param $links
     *
     * @return array
     */
    protected function applyActionLinks( $links ) {
        return array_merge( $this->actionlinks, $links );
    }
    
    /**
     * Plugin activation
     */
    protected function activate() {
        foreach ( $this->components_pages as $components_page ) {
            $components_page->activateComponents();
        }
    }
    
    /**
     * Plugin deactivation
     */
    protected function deactivate() {
        $filtered_plugins = static::getAllPlugins();
        
        foreach ( $this->components_pages as $components_page ) {
            $components_page->deactivateComponents();
            $filtered_plugins = $components_page->getFilteredPlugins($filtered_plugins);
        }
        
        ObjectDB::update( 'active_plugins', $filtered_plugins );
    }
}